function [C_e_b_nova, v_e_eb_nova, r_e_eb_nova, IMU_bias_nou, P_nova] = ...
    filtreKalman_I_MeasurementUpdate_GPS( P_propagada, x_propagada,...
    C_e_b_vella, r_e_eb_vella,  v_e_eb_vella, GNSS_r_eb_e, IMU_bias_vell)
%FILTREKALMAN_I_MEASUREMENTUPDATE_GPS: Executa l'etapa de measurement
%update del filtre de Kalman. Fa un Update de posici amb les dades que
%arriben d'un GPS.
%Inputs:
%       x_propagada: vector d'estat.
%       C_e_b_vella: Matriu de rotaci de body a ECEF.
%       w_b_ib: velocitat angular que llegeig la IMU.
%       r_e_eb_vella: posici en coordenades ECEF (m).
%       v_e_eb_vella: velocitat en coordenades ECEF (m/s).
%       P_propagada: matriu de covariana anterior.
%       GNSS_r_eb_e: Posici GPS o zero update.
%       IMU_bias_vell: Estimaci del bias de la IMU (accelermetres i
%       giroscopis)
%Outputs:
%       C_e_b_nova: Estimaci de la matriu de rotaci de body a ECEF
%       v_e_eb_nova: Estimaci de la velocitat en coordenades ECEF (m/s)
%       r_e_eb_nova: Estimaci de la posici en coordenades ECEF (m/s)
%       IMU_bias_nou: Estimaci del bias de la IMU.
%                files 1-3 bias accelermetres (m/s^2).
%                files 4-6 bias giroscopis (rad/s).
%       P_nova: Covariance matrix
%MESURAMENT MODEL:
    
    %PAS 5: Mesurament matrix.
    
    H = zeros(3, 15);
    H(1:3,7:9) = -eye(3);
       
    %PAS 6: Mesurament noise covariance matrix
    
    LC_KF_pos_meas_SD = 2.5;
    LC_KF_vel_meas_SD = 0.1;
   
    R(1:3,1:3) = eye(3) * LC_KF_pos_meas_SD;
        
    %PAS 7: K de Kalaman
    
    K = P_propagada*H'*inv(H*P_propagada*H' + R);
    
    %PAS 8: Calcular z.
    
    delta_z(1:3,1) = GNSS_r_eb_e - r_e_eb_vella;
    
    %PAS 9: Estimaci de xk+ a partir de xk-.
    
    x_nova = x_propagada + K*delta_z;
    
    %PAS 10: Estimaci de Pk- a Pk+.
    
    P_nova = (eye(15) - K*H)*P_propagada;
    
    % CORRECCI CLOSED-LOOP 

    % Correccions d'orientaci, velocitat i posici
    C_e_b_nova = (eye(3) - Skew_symmetric(x_nova(1:3))) * C_e_b_vella;
    v_e_eb_nova = v_e_eb_vella - x_nova(4:6);
    r_e_eb_nova = r_e_eb_vella - x_nova(7:9);

    % Estimaci del bias de la IMU
    IMU_bias_nou = IMU_bias_vell + x_nova(10:15);
    
end

