function [g] = gravetat_ECI(rx, ry, rz)
%Dona la gravetat segons el punt de la Terra on es troba
%Inputs:
%       rx, ry, rz: posici en coordenades ECI (m).
%Outputs:
%       g: gravetat (m/s^2).

    R0 = 6378137;
    mu = 3.986004418E14;
    J_2 = 1.082627E-3;
    norma = sqrt(rx^2 + ry^2 + rz^2);
    
    r_i_ib = [rx; ry; rz];
    
    vector_calcul_x = (1 - 5*(rz/norma)^2)*rx;
    vector_calcul_y = (1 - 5*(rz/norma)^2)*ry;
    vector_calcul_z = (3 - 5*(rz/norma)^2)*rz;
    
    vector_calcul = [vector_calcul_x; vector_calcul_y; vector_calcul_z];
    
    gamma = -mu/norma^3*(r_i_ib + (3/2)*J_2*(R0^2/norma^2)*vector_calcul);
    
    g = gamma;

end

