%
%               PROGRAMA ECEF
%
% Script que calcula la posici, velocitat, orientaci i acceleraci de la
% IMU segons el sistema de referncia ECEF.

%               Inicialitzaci

clc;
clear all;

%Carraguem dades de la IMU
[ indat ] = data_loader( 'voltagran1.csv' );
matriuDades = indat;

%Carraguem dades del GPS
addpath('voltacotxe/')
addpath('NMEAReader/')

[hour, minute, second, velocity, numsats, height, lat_decimal,...
    long_decimal, quality] = readGPS('GPS1.TXT');

%Transformar les dades dels griscopis de graus/s a rad/s. I clcul del
%bias.
matriuDades(:,2:4)=matriuDades(:,2:4)*pi/180;

%Temps d'iteraci (s) i velocitat de rotaci de la Terra (rad/s).
w_ie = 7.292115e-5;
fila = 1;
cops = 0;

t_old = 0;
[ t, label, indexs ] = samplingTimeVoltaGran( matriuDades );

 % Troba Roll, Pitch i Yaw inicials de la IMU
[angleFi_nb, angleTeta_nb, angleFi_nb_graus, angleTeta_nb_graus] = ...
    leveling(matriuDades); 
[anglePsi_nb, anglePsi_nb_graus] = magneticHeading(matriuDades); 

%Posici segons parmetres de latitud, longitud i altitud (en graus).
L = lat_decimal(1647,1);
lambda = long_decimal(1647,1);
h = height(1647,1);

L_rad = degtorad(L);
lambda_rad = degtorad(lambda);

%Troba posici respecte ECEF
[rx,ry,rz] = posicioNED_ECEF(L_rad, lambda_rad, h);                      
R_e_eb_vella = [rx; ry; rz];
GNSS_r_e_eb = [rx; ry; rz];
PosicioInicial = [rx; ry; rz];

%Creem una matriu de rotaci de Body a NED amb els valors dels angles
%trobats amb les funcions de leveling i magnetic Heading. Desprs fem la
%matriu de rotaci de NED a ECEF amb els valors de la posici (L, lambda).
%Finalment, multipliquem les dues matrius calculades i trobem la que va de
%Body a ECEF.
[C_ned_b] = crearMatriuRotacioBody_NED( angleFi_nb, angleTeta_nb, ...
    anglePsi_nb);      
[C_ecef_ned] = crearMatriuRotacioNED_ECEF( L, lambda);                               
C_ecef_b_vella = C_ecef_ned*C_ned_b;                                                       

%Velocitat inicial 0.                                       
V_e_eb_vella = [0; 0; 0];
GNSS_v_e_eb = [0; 0; 0];

%Inicialitzaci de dades pel filtre de Kalman (matriu P i bias).
LC_KF_orientacio = degtorad(1);
LC_KF_velocitat = 0.1;
LC_KF_posicio = 10;
LC_KF_biasA = 1000 * 9.80665E-6;
LC_KF_biasG = 10 * 0.01745329252 / 3600;

P =  zeros(15);
P(1:3,1:3) = eye(3) * LC_KF_orientacio^2;
P(4:6,4:6) = eye(3) * LC_KF_velocitat^2;
P(7:9,7:9) = eye(3) * LC_KF_posicio^2;
P(10:12,10:12) = eye(3) * LC_KF_biasA^2;
P(13:15,13:15) = eye(3) * LC_KF_biasG^2;

IMU_bias_vell = zeros(6,1);                     

%************ITERACI****************
for i=2:length(t),
    
    At = t(i) - t_old;
    
    if label(i)==1
        %Llegim el valor dels girscops.
        [wx_b_ib, wy_b_ib, wz_b_ib] = llegirW(indexs(i), matriuDades);
        W_b_ib = [wx_b_ib; wy_b_ib; wz_b_ib];   

        %Llegim la fora especfica
        [fx_b_ib, fy_b_ib, fz_b_ib] = ...
            llegirForcaEspecifica(indexs(i),matriuDades);
        F_b_ib = [fx_b_ib; fy_b_ib; fz_b_ib]*9.80665;

        %Correcci bias
        F_b_ib = F_b_ib - IMU_bias_vell(1:3);
        W_b_ib = W_b_ib - IMU_bias_vell(4:6);

        %   EQUACIONS DE NAVEGACIO ECEF
        %Apliquem les equacions de navegaci per trobar la posici, la
        %velocitat i la matriu de rotacio.
        [ R_e_eb, V_e_eb, C_ecef_b] = equacionsNavegacioECEF(At,...
            R_e_eb_vella, V_e_eb_vella, C_ecef_b_vella, F_b_ib, W_b_ib );

        PosicioECEF(fila, 1) = R_e_eb(1,1)- PosicioInicial(1,1);
        PosicioECEF(fila, 2) = R_e_eb(2,1)- PosicioInicial(2,1);
        PosicioECEF(fila, 3) = R_e_eb(3,1)- PosicioInicial(3,1);
        VelocitatECEF(fila, 1:3) = V_e_eb';
        fila = fila + 1;
    end
    
    if label(i)==1 && t(i)<10
        
        %Etapa de Predicci del Filtre de Kalman
         [P, x_propagada] = filtreKalman_I_Prediccio( At,...
         C_ecef_b_vella, F_b_ib, R_e_eb_vella, P);

         %Measurment Update del Filtre de Kalman amb Zupdates de
         %velocitat lineal i angular.
         [C_ecef_b,  V_e_eb, R_e_eb, IMU_bias, P  ] = ...
         filtreKalman_I_MeasurementUpdate_ZU( P, x_propagada,...
         C_ecef_b_vella,...
         W_b_ib,  R_e_eb_vella,  V_e_eb_vella, IMU_bias_vell);
    end
    
    if label(i)==2 && t(i)<10
        
        %Llegim les coordenades del GPS
         L_rad = degtorad(lat_decimal(indexs(i),1));
         lambda_rad = degtorad(long_decimal(indexs(i),1));
         h = height(indexs(i),1);

         %Passem les coordenades de GPS a ECEF
         [rx,ry,rz] = posicioNED_ECEF(L_rad, lambda_rad, h);                      
         GNSS_r_e_eb = [rx; ry; rz];

         %Etapa de Predicci del Filtre de Kalman
         [P, x_propagada] = filtreKalman_I_Prediccio( At,...
         C_ecef_b_vella, F_b_ib, R_e_eb_vella, P);

         %Measurement Update del Filtre de Kalman amb Zupdates de
         %velocitat lineal i angular i amb dades del GPS
         [C_ecef_b,  V_e_eb, R_e_eb, IMU_bias, P  ] = ...
         filtreKalman_I_MeasurementUpdate_GPS_ZU( P, x_propagada,...
         C_ecef_b_vella, W_b_ib,  R_e_eb_vella,  V_e_eb_vella,...
         GNSS_r_e_eb, IMU_bias_vell);
         
    end
    
    if t(i)>10
        
         %Etapa de Predicci del Filtre de Kalman
         [P, x_propagada] = filtreKalman_I_Prediccio( At,...
         C_ecef_b_vella, F_b_ib, R_e_eb_vella, P);
     
    end
    
    
    if label(i)==2 && t(i)>10 
       
        %Llegim les coordenades del GPS
         L_rad = degtorad(lat_decimal(indexs(i),1));
         lambda_rad = degtorad(long_decimal(indexs(i),1));
         h = height(indexs(i),1);

         %Passem les coordenades de GPS a ECEF
         [rx,ry,rz] = posicioNED_ECEF(L_rad, lambda_rad, h);                      
         GNSS_r_e_eb = [rx; ry; rz];
         
         %Measurement Updates del filtre de Kalman amb Updates del GPS
         [C_ecef_b,  V_e_eb, R_e_eb, IMU_bias, P  ] = ...
        filtreKalman_I_MeasurementUpdate_GPS( P, x_propagada,...
        C_ecef_b_vella, R_e_eb_vella,  V_e_eb_vella, GNSS_r_e_eb,...
        IMU_bias_vell);
    end

    IMU_bias_vell = IMU_bias;
    t_old = t(i);
    cops = cops + 1;

    %   CLCUL ANGLES EULER (Roll, Pitch i Yaw)
    
    %Passem la posici trobada respecte ECEF a NED frame.
    [L, lambda, h] = Borkowski(R_e_eb(1), R_e_eb(2), R_e_eb(3));
    
    %Trobem matriu de rotaci de ECEF a NED.
    [ C_ned_ecef ] = crearMatriuRotacioECEF_NED( L, lambda);
    
    %Trobem matriu de Body a NED.
    C_n_b = C_ned_ecef*C_ecef_b;
    
    %Trobem angles d'Euler.
    [Roll, Pitch, Yaw] = calcularAnglesEuler( C_n_b );
    
    AnglesEuler(fila, 1:3)=[Roll; Pitch; Yaw];
    
   
    R_e_eb_vella = R_e_eb;
    V_e_eb_vella = V_e_eb;
    C_ecef_b_vella = C_ecef_b;
    
end

