%
%               PROGRAMA NED
%
% Script que calcula la posici, velocitat, orientaci i acceleraci de la
% IMU segons el sistema de referncia NED.

%               Inicialitzaci

clc;
clear all;
load('matlab.mat');

[ indat ] = data_loader( 'quiet2.csv' );
matriuDades = indat;

%Transformar les dades dels griscopis de graus/s a rad/s. I clcul del
%bias.
matriuDades(:,2:4)=matriuDades(:,2:4)*pi/180;
%b_gir=mean(matriuDades(:,2:4))';

%Temps d'iteraci (s), velocitat rotaci de la Terra (rad/s) i fila.
t = 1/102.5;                                                        
w_ie = 7.292115e-5;                                             

%Troba Roll i Pitch de la IMU
[angleFi_nb, angleTeta_nb, angleFi_nb_graus, angleTeta_nb_graus] = leveling(matriuDades);  

%Troba Yaw de la IMU
[anglePsi_nb, anglePsi_nb_graus] = magneticHeading(matriuDades); 

%Posici segons parmetres de latitud, longitud i altitud (en graus).
L_graus = input('Ingresa el valor de la latitud (L) inicial: ');            
lambda_graus = input('Ingresa el valor de la longitud inicial: ');
h_vella = input('Ingresa el valor de la altitud inicial: ');

%Passem la longitud i la latitud a radians
L_vella = degtorad(L_graus);
lambda_vella = degtorad(lambda_graus);

%Creem matriu de rotaci de body a NED frame
[C_ned_b_anterior] = crearMatriuRotacioBody_NED( angleFi_nb, angleTeta_nb, anglePsi_nb);      

%Velocitat inicial respecte NED frame
V_n_eb_vella = [0; 0; 0];

%Creem fitxers de dades
fileID = fopen('resultatsProvaQuiet_NED4.txt','w');     
fprintf(fileID,'\n%s %s %s\n\n','L','lambda','h');
fprintf(fileID,'%f      %f      %f\n', L_graus, lambda_graus, h_vella);

%Creem fitxer de dades amb coordenades ECEF per poder comparar els
%resultats.
[x, y, z] = posicioNED_ECEF(L_vella, lambda_vella, h_vella);

fileID2 = fopen('resultatsProvaQuiet_NED_ECEF4.txt','w');     
fprintf(fileID2,'\n%s %s %s\n\n','x','y','z');
fprintf(fileID2,'%f      %f      %f\n', x, y, z);

%Creem fitxers de dades per guardar angles d'Euler.
fileID3 = fopen('resultatsProvaQuiet_NED_orientacio.txt','w');     
fprintf(fileID3,'\n%s %s %s\n\n','Roll','Pitch','Yaw');
fprintf(fileID3,'%f      %f      %f\n', angleFi_nb_graus, angleTeta_nb_graus, anglePsi_nb_graus);

%Ens dona el nombre de files (n) i columnes (c) de la matriu de dades de la IMU.
[n, c] = size(matriuDades);                           

%*********ITERACI************


for fila=1:1:n,
    
   
    %Agafem velocitat angualr de la matriu de dades.
    [wx_b_ib, wy_b_ib, wz_b_ib] = llegirW(fila, matriuDades);
    W_b_ib = [wx_b_ib; wy_b_ib; wz_b_ib];   
    
    %Llegim la fora especfica
    [fx_b_ib, fy_b_ib, fz_b_ib] = llegirForcaEspecifica(fila,matriuDades);
    F_b_ib = [fx_b_ib; fy_b_ib; fz_b_ib]*9.80665;
    
    %   EQUACIONS DE NAVEGACIO NED
    %Apliquem les equacions de navegaci per trobar la posici, la
    %velocitat i la matriu de rotacio.
    
    [L, lambda, h, V_n_eb, C_ned_b] = equacionsNavegacioNED(t, L_vella,...
        lambda_vella, h_vella, V_n_eb_vella, C_ned_b_anterior, F_b_ib, W_b_ib );
    
          
    %Passem a graus per grabar-ho al fitxer en coordenades NED i desprs ho
    %passem a ECEF per guardar-ho en un fitxer amb coordenades ECEF per
    %poder comparar.
    L_graus = radtodeg(L);
    lambda_graus = radtodeg(lambda);
    fprintf(fileID,'%f      %f    %f\n', L_graus, lambda_graus, h);
    
    [x, y, z] = posicioNED_ECEF(L, lambda, h);
    fprintf(fileID2,'%f      %f    %f\n', x, y, z);
    R_n_eb = [ x; y; z];
    
        
     %Creem matriu amb els resultats de la posici i velocitat.
    PosicioNEDambECEF(fila, 1) = R_n_eb(1,1)- 4743050.930637;
    PosicioNEDambECEF(fila, 2) = R_n_eb(2,1)- 252319.376959;
    PosicioNEDambECEF(fila, 3) = R_n_eb(3,1)- 4242591.914783;
    Velocitat(fila, 1:3) = V_n_eb';
    
    %Grabar orientaci en un fitxer.
    [Roll, Pitch, Yaw] = calcularAnglesEuler( C_ned_b );
    fprintf(fileID3,'%f      %f    %f\n', Roll, Pitch, Yaw);
    
    %Actualitzem C_ned_b_anterior, L_vella, lambda_vella, h_vella,
    %V_n_eb_vella
    C_ned_b_anterior = C_ned_b;
        
    L_vella = L;
    lambda_vella = lambda;
    h_vella = h;
    V_n_eb_vella = V_n_eb;
    
end

fclose(fileID);
fclose(fileID2);
fclose(fileID3);

